#include "StdAfx.h"
#include "NMEA2.h"
#include "gps.h"

CGPS::CGPS(void)
: m_Data(NULL)
{
	bPort=1; 
	dwBaudRate=9600; 
	bByteSize=8; 
	bStopBits=1; 
	bParity=0; 

	m_Data = new char[255];
	memset(m_Data, 0, sizeof(char) * 255);

	strcpy(m_seps,",");
}

CGPS::CGPS(int ComPort, DWORD BaudRate, BYTE ByteSize, BYTE Parity, BYTE StopBits)
: m_Data(NULL)
{
	bPort=ComPort; 
	dwBaudRate=BaudRate; 
	bByteSize=ByteSize; 
	bStopBits=StopBits; 
	bParity=Parity; 

	m_Data = new char[255];
	memset(m_Data, 0, sizeof(char) * 255);

	strcpy(m_seps,",");
}

CGPS::~CGPS(void)
{
	delete [] m_Data;
}

int CGPS::PutRawData(const char* Data)
{
	ParseData(Data);

	return 0;
}

int CGPS::SetupComPort(int ComPort, DWORD BaudRate, BYTE ByteSize, BYTE Parity, BYTE StopBits)
{
	bPort=ComPort; 
	dwBaudRate=BaudRate; 
	bByteSize=ByteSize; 
	bStopBits=StopBits; 
	bParity=Parity;

	return 0;
}

int CGPS::ParseData(const char* Data)
{
	char* t_Data;
	char* t_Header;
	int t_flag = 0;

	t_Data = new char[255];
    
	memset(t_Data, 0 , sizeof(char) * 255);
	memcpy(t_Data, Data, sizeof(char) * strlen(Data));

	t_Header = strtok(t_Data, m_seps);

	if(t_Header == NULL)
		return -1;

	for(int i=3;i<6;i++){
		t_flag += (int)(*(t_Header+i));
	}

	memset(t_Data, 0 , sizeof(char) * 255);
	memcpy(t_Data, Data, sizeof(char) * strlen(Data));

	switch(t_flag){
		case 0xdf:
			SetGLLData(t_Data);
			break;
		case 0xcf:
			SetGGAData(t_Data);
			break;
		case 0xdb:
//			SetGSAData(t_Data);
			break;
		case 0xe2:
//			SetRMCData(t_Data);
			break;
		case 0xf1:
//			SetVTGData(t_Data);
		break;
	}
		
	delete [] t_Data;

	return 0;
}

void CGPS::SetGLLData(char* Data)
{
	char* t_token;

	t_token = strtok(Data, m_seps);
	if(strcmp(t_token,"$GPGLL"))
		return ;

	strcpy(m_GLL.MessageID, "GLL");
	
	t_token = strtok(NULL, m_seps);
	m_GLL.Latitude = a2f(t_token);

	t_token = strtok(NULL, m_seps);
	m_GLL.NSIndicator = *t_token;

	t_token = strtok(NULL, m_seps);
	m_GLL.Logitude = a2f(t_token);
	
	t_token = strtok(NULL, m_seps);
	m_GLL.EWIndicator = *t_token;

	t_token = strtok(NULL, m_seps);
	m_GLL.UTCPosition = a2f(t_token);	

	t_token = strtok(NULL, m_seps);
	m_GLL.Status = *t_token;
	
/*	t_token = strtok(NULL, m_seps);
	m_GLL.Checksum = atoi(t_token);*/
}

void CGPS::SetGGAData(char* Data)
{
	char* t_token;

	t_token = strtok(Data, m_seps);

	strcpy(m_GGA.MessageID, "GGA");

	t_token = strtok(NULL, m_seps);
	m_GGA.UTCPosition = a2f(t_token);

	t_token = strtok(NULL, m_seps);
	m_GGA.Latitude = a2f(t_token);

	t_token = strtok(NULL, m_seps);
	m_GGA.NSIndicator = *t_token;

	t_token = strtok(NULL, m_seps);
	m_GGA.Longitude = a2f(t_token);

	t_token = strtok(NULL, m_seps);
	m_GGA.EWIndicator = *t_token;

	t_token = strtok(NULL, m_seps);
	m_GGA.PositionFixIndicator = a2c(t_token);

	t_token = strtok(NULL, m_seps);
	m_GGA.SatelliteUsed = a2i(t_token);

	t_token = strtok(NULL, m_seps);
	m_GGA.HDOP = a2f(t_token);

	t_token = strtok(NULL, m_seps);
	m_GGA.MSLAltitude = a2f(t_token);

	t_token = strtok(NULL, m_seps);
//	m_GGA.Units1 = *t_token;

	t_token = strtok(NULL, m_seps);
	m_GGA.GeoidSeperation = a2f(t_token);

	t_token = strtok(NULL, m_seps);
//	m_GGA.Units2 = *t_token;

/*	t_token = strtok(NULL, m_seps);			fields for DGPS

	t_token = strtok(NULL, m_seps);*/
}

void CGPS::SetGSAData(char* Data)
{
}

void CGPS::SetRMCData(char* Data)
{
}

void CGPS::SetVTGData(char* Data)
{
}

double CGPS::a2f(const char* Data)
{
	__try{
		if(strlen(Data) < 1)
			return .0;
		else
			return atof(Data);
	}__except(EXCEPTION_EXECUTE_HANDLER){
		return .0;
	}

}

int CGPS::a2i(const char* Data)
{
	__try{
		if(strlen(Data) < 1)
			return 0;
		else
			return atoi(Data);
	}__except(EXCEPTION_EXECUTE_HANDLER){
		return 0;
	}

}

char CGPS::a2c(const char* Data)
{
	__try{
		if(strlen(Data) < 1)
			return 0;
		else
			return *Data;
	}__except(EXCEPTION_EXECUTE_HANDLER){
		return 0;
	}

}

char CGPS::GetNMEAChecksum(const char* Data)
{
	char Checksum = 0;
	int i=0;

	Checksum = *(Data + i++);

	do{
		if (*Data != ',')
			Checksum ^= *(Data + i++);
	}while(*(Data + i++));

	return Checksum;
}
